import base64
import os
from flask import Flask, request, render_template
from tensorflow import keras
from PIL import Image
import numpy as np
from flask import redirect, url_for


 

app = Flask(__name__)

 

app.static_folder = 'static'

 

@app.route('/')

def index():

    return render_template('index.html')



# Load the binary classification model
try:
    binary_model = keras.models.load_model('C:/Users/Beuni/Desktop/research/model_full_binary_fungi_model.h5')
    print("Binary Model loaded successfully.")
except Exception as e:
    print("Error loading the binary model:", str(e))

# Load the multi-class classification model
try:
    multi_class_model = keras.models.load_model('C:/Users/Beuni/Desktop/research/model_full_multi_fungi_model.h5')
    print("Multi-Class Model loaded successfully.")
except Exception as e:
    print("Error loading the multi-class model:", str(e))

# Define a function to make binary predictions
def predict_binary_image(image_path):
    img = Image.open(image_path)
    img = img.resize((128, 128))
    img_array = np.array(img)
    img_array = np.expand_dims(img_array, axis=0)
    img_array = img_array / 255.0  # Normalize pixel values to [0, 1]
    
    # Make a binary prediction
    prediction = binary_model.predict(img_array)
    
    if prediction <= 0.5:
        result = "Fungus type"
    else:
        result = "Not a fungus"
    
    return result

# Define a function to make multi-class predictions
def predict_multi_class_image(image_path):
    img = Image.open(image_path)
    img = img.resize((128, 128))
    img_array = np.array(img)
    img_array = np.expand_dims(img_array, axis=0)
    img_array = img_array / 255.0  # Normalize pixel values to [0, 1]
    
    # Make a multi-class prediction
    prediction = multi_class_model.predict(img_array)

    if prediction == "Mycorrhizal Fungi":
        result = "Mycorrhizal type"
    elif prediction  == "Aspergillus spp. ":
        result = "Aspergillus a fungus"
    else:
        result = "Candida fungus"
    
    
    
    # Define your class labels based on your model's classes
    class_labels = ["Mycorrhizal Fungi", "Aspergillus spp. ", "Candida spp. "]  
    
    # Extract the class with the highest probability
    predicted_class_index = np.argmax(prediction, axis=1)
    result = class_labels[predicted_class_index[0]]
    
    return result

@app.route('/index3', methods=['GET', 'POST'])
def index1():
    image_data = None
    binary_prediction_result = None
    multi_class_prediction_result = None

    if request.method == 'POST':
        # Check if a file was uploaded
        if 'file' not in request.files:
            return render_template('index3.html', message='No file part')

        file = request.files['file']

        # Check if the file has a filename
        if file.filename == '':
            return render_template('index3.html', message='No selected file')

        # Check if the file is allowed
        allowed_extensions = {'jpg', 'jpeg', 'png'}
        if not allowed_file(file.filename, allowed_extensions):
            return render_template('index3.html', message='Invalid file extension')

        # Save the uploaded file to a temporary location
        file_path = os.path.join('uploads', file.filename)
        file.save(file_path)

        # Make a binary prediction using the model
        binary_prediction_result = predict_binary_image(file_path)

        # If the image is identified as a fungus, make a multi-class prediction
        if binary_prediction_result == "Fungus type":
            multi_class_prediction_result = predict_multi_class_image(file_path)

        # Encode the uploaded image as base64
        with open(file_path, "rb") as image_file:
            image_data = base64.b64encode(image_file.read()).decode("utf-8")

        # Delete the uploaded file
        os.remove(file_path)


    
    return render_template('index3.html', image_data=image_data, binary_prediction=binary_prediction_result, multi_class_prediction=multi_class_prediction_result)

def allowed_file(filename, allowed_extensions):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in allowed_extensions

if __name__ == "__main__":
    app.run(debug=True)



